package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.*;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.diagbot.api.ImportExcel;
import com.diagbot.dto.ConceptDTO;
import com.diagbot.dto.LibraryInfoDTO;
import com.diagbot.entity.*;
//import com.diagbot.dto.RelationDTO;
//import com.diagbot.exception.CommonErrorCode;
//import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.ConceptServiceImpl;
import com.diagbot.vo.AmendTermVo;
//import com.diagbot.vo.MedicalVo;
import com.diagbot.vo.TermVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.multipart.MultipartFile;


import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * @Description: 术语查询业务层
 * @author: Weixuan Huang
 * @time: 2019/1/14 16:17
 */
@Component
public class ConceptFacade extends ConceptServiceImpl {

    @Autowired
    private LibraryInfoFacade libraryinfoFacade;
    @Autowired
    private LibraryDetailFacade libraryDetailFacade;
    @Autowired
    private MedicalFacade medicalFacade;
    @Autowired
    private RelationFacade relationFacade;


    /**
     * 获取医学术语基本信息
     *
     * @param termvo
     * @return List<ConceptDTO>
     */
    public List<ConceptDTO> getConceptInfo(@RequestBody TermVo termvo) {

        List<ConceptDTO> conceptDTOs = new ArrayList<>();
        ConceptDTO conceptDTO;

        // 获取医学术语的基本信息
        QueryWrapper<LibraryInfo> query = new QueryWrapper<>();
        query.like("name", termvo.getTerm());
        query.eq("is_concept", 1);
        List<LibraryInfo> libraryInfos = libraryinfoFacade.list(query);

        for (LibraryInfo libraryInfo:libraryInfos) {
            conceptDTO = new ConceptDTO();
            conceptDTO.setId(libraryInfo.getConceptId());
            conceptDTO.setLibId(libraryInfo.getId().intValue());
            conceptDTO.setName(libraryInfo.getName());
            conceptDTO.setLibraryInfo(libraryInfo);

            conceptDTOs.add(conceptDTO);
        }

        return conceptDTOs;
    }


    /**
     * 获取医学术语静态信息
     *
     * @param termvo
     * @return ConceptDTO
     *
    public ConceptDTO getConceptKnowledge(@RequestBody TermVo termvo) {

        LibraryInfo libraryInfo;
        List<LibraryDetail> libraryDetails;
        TermVo termVo;

        ConceptDTO conceptDTO = getConcept(termvo);

        if (conceptDTO != null) {
            // 获取与医学术语的基本信息
            QueryWrapper<LibraryInfo> query = new QueryWrapper<>();
            query.eq("concept_id", conceptDTO.getId());
            query.eq("is_concept", 1);
            libraryInfo = libraryinfoFacade.getLibraryInfor(query);

            if (libraryInfo != null) {
                conceptDTO.setName(libraryInfo.getName());

                // 获取与术语相关的静态知识
                termVo = new TermVo();
                termVo.setConcept_id(conceptDTO.getId());
                termVo.setTerm(libraryInfo.getName());
                termVo.setType(libraryInfo.getType());

                libraryDetails = libraryDetailFacade.getLibraryDetail(termVo);

                if (libraryDetails != null) {
                    conceptDTO.setInformation(libraryDetails);
                }
            }
        }


        return conceptDTO;
    }



    /**
     * 获取术语医学信息
     *
     * @param termvo
     * @return ConceptDTO
     *
    public ConceptDTO getConceptMedInfo(@RequestBody TermVo termvo) {
        LibraryInfo libraryInfo;

        ConceptDTO conceptDTO = getConcept(termvo);

        if (conceptDTO != null) {
            // 获取与医学术语的基本信息
            QueryWrapper<LibraryInfo> query = new QueryWrapper<>();
            query.eq("concept_id", conceptDTO.getId());
            query.eq("is_concept", 1);
            libraryInfo = libraryinfoFacade.getLibraryInfor(query);

            if (libraryInfo != null) {
                conceptDTO.setName(libraryInfo.getName());
                // 获取与术语相关的临床医学信息
                QueryWrapper<Medical> medquery = new QueryWrapper<>();
                query.eq("concept_id", conceptDTO.getId());
                query.eq("name", termvo.getTerm());
                conceptDTO.setMedicalInfo(medicalFacade.getOne(medquery));
            }
        }

        return conceptDTO;
    }

    /**
     * 获取术语关系信息
     *
     * @param termvo
     * @return ConceptDTO
     *
    public ConceptDTO getConceptRelation(@RequestBody TermVo termvo) {
        LibraryInfo libraryInfo;
        TermVo termVo;

        ConceptDTO conceptDTO = getConcept(termvo);
        // 获取医学术语的基本信息
        if (conceptDTO != null) {
            // 获取与医学术语的基本信息
            QueryWrapper<LibraryInfo> query = new QueryWrapper<>();
            query.eq("concept_id", conceptDTO.getId());
            query.eq("is_concept", 1);
            libraryInfo = libraryinfoFacade.getLibraryInfor(query);

            if (libraryInfo != null) {
                conceptDTO.setName(libraryInfo.getName());

                termVo = new TermVo();
                termVo.setConcept_id(conceptDTO.getId());

                // 获取术语的所有上级信息
                conceptDTO.setRelations(relationFacade.getAllRelation(termvo));
            }
        }

        return conceptDTO;
    }
    */

    /**
     * 获取医学术语列表
     *
     * @param page
     * @return IPage<LibraryDTO>
     */
    public IPage<LibraryInfoDTO> getAllConcepts(Page page) {
        LibraryInfoDTO libraryInfoDTO = new LibraryInfoDTO();

        return libraryinfoFacade.getAllConcept(page, libraryInfoDTO);
    }


        /**
         * 更新医学术语信息
         *
         * @param amendTermVo
         * @return ConceptDTO
         */
    public ConceptDTO upsertConceptInfo(@RequestBody AmendTermVo amendTermVo) {
        ConceptDTO conceptDTO = new ConceptDTO();

        TermVo termVo = new TermVo();
        termVo.setTerm(amendTermVo.getConcept());
        termVo.setType(amendTermVo.getType());

        // 如果概念术语不存在，添加新术语，然后添加新的概念
        conceptDTO = getConcept(termVo);
        if (conceptDTO == null) {
            AmendTermVo amendconcept = new AmendTermVo();
            amendconcept.setOldterm(amendTermVo.getOldterm());
            amendconcept.setNewterm(amendTermVo.getConcept());
            amendconcept.setConcept("");
            amendconcept.setType(amendTermVo.getType());
            libraryinfoFacade.upsertLibraryInfo(amendconcept);

            Map<String, Object> params = new HashMap<>();
            params.put("name", amendTermVo.getConcept());
            params.put("type", amendTermVo.getType());
            int libid = libraryinfoFacade.getLibraryInfor(params).getId().intValue();


            Concept concept = new Concept();
            concept.setLibId(libid);
            concept.setGmtCreated(LocalDateTime.now());
            this.saveOrUpdate(concept);

            termVo.setType(amendTermVo.getType());
            conceptDTO = getConcept(termVo);
        }

        amendTermVo.setConcept_id(conceptDTO.getId());
        LibraryInfo libraryInfo = libraryinfoFacade.upsertLibraryInfo(amendTermVo);
        conceptDTO.setLibraryInfo(libraryInfo);

        conceptDTO = updateConceptInfo(amendTermVo, conceptDTO);

        return conceptDTO;
    }

    /**
     * 更新或添加医学术语信息
     *
     * @param amendTermVo
     * @param conceptDTO
     * @return ConceptDTO
     */
    public ConceptDTO updateConceptInfo(AmendTermVo amendTermVo, ConceptDTO conceptDTO) {

        libraryDetailFacade.upsertLibraryInfo(amendTermVo);
        conceptDTO.setInformation(amendTermVo.getInformation());

        medicalFacade.upsertMedicalInfo(amendTermVo);
        conceptDTO.setMedicalInfo(amendTermVo.getMedicalInfo());

        relationFacade.upsertRelationInfo(amendTermVo);
        conceptDTO.setRelations(amendTermVo.getRelations());

        return conceptDTO;
    }



    /**
     * 获取概念Id
     *
     * @param term
     * @return Integer
     */
    public int getConceptId(String term, String type) {
        int id = 0;
        TermVo termVo = new TermVo();

        try {
            termVo.setTerm(term);
            termVo.setType(type);
            ConceptDTO conceptDTO = getConcept(termVo);
            if (conceptDTO != null) {
                id = conceptDTO.getId();
            }
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
        finally {
            return id;
        }
    }

    /**
     * 从文件批量导入术语信息
     *
     * @param file
     * @return ConceptDTO
     */
    public ConceptDTO importBatch(MultipartFile file) {
        ImportExcel.setConceptFacade(this);
        ImportExcel.setLibraryinfoFacade(libraryinfoFacade);
        ImportExcel.setLibraryDetailFacade(libraryDetailFacade);
        ImportExcel.setMedicalFacade(medicalFacade);
        ImportExcel.setRelationFacade(relationFacade);

        ConceptDTO conceptDTO = ImportExcel.importBatch(file);

        return conceptDTO;
    }


}
